/**
 ******************************************************************************
 * @project  Arduino TCP Loopback LED ControlExample 
 * @author   Edward Ahn
 * @version  V1.1
 * @date     11-May-2016
 * @brief    Ethernet Communication
 * @MCU      ARM Cortex-M0+
*******************************************************************************
**/

#include <SPI.h>
#include <Ethernet2.h> // Ethernet 라이브러리를 보면 SPI를 사용하여 아두이노와 통신한다.

#if defined(WIZ550io_WITH_MACADDRESS) // WIZ550io를 사용할 경우 MAC이 필요 없음.
;
#else // WIZ550io가 아니라면 MAC을 설정해주어야 한다.
byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED}; // MAC ADDRESS.
#endif

#define __USE_DHCP__ // DHCP사용 시 define

IPAddress ip(192,168,1, 177); // TCP/IP 네트워크에 있는 호스트를 식별하는 숫자
IPAddress gateway(192,168,1, 1); // 다른 네트워크에 있는 호스트와 통신할 때 사용
IPAddress subnet(255, 255, 255, 0); // 호스트가 로컬 or 원격 네트워크에 있는지 확인할 때 사용
IPAddress myDns(8, 8, 8, 8); // google puble dns

// 네트워크 정보에 대한 자세한 내용은 아래 사이트 확인 
// https://support.microsoft.com/ko-kr/kb/164015

// 소켓 생성할 때 5000번 포트로 생성
// 사용자가 원하는 대로 IP 및 포트 설정 가능
EthernetServer server(5000);
// 버퍼 클리어를 위한 Flag 선언
// 이전에 연결되었던 클라이언트와의 데이터가 버퍼에 남아있을 수 있으니 버퍼 클리어를 진행
boolean alreadyConnected = false;
String buffer = ""; // 문자열 버퍼 선언

#define LED_PORT 13    // LED PORT 번호 정의

void setup() {
// 이더넷 디바이스 initialize
#if defined __USE_DHCP__ // DHCP 사용 시 (네트워크 정보 DHCP에서 할당받음)
#if defined(WIZ550io_WITH_MACADDRESS) // WIZ550io에 할당된 MAC 주소를 사용할 경우
  Ethernet.begin();
#else
  Ethernet.begin(mac); // MAC이 없는 이더넷 디바이스의 경우
#endif
#else // Static 주소로 사용 시 (네트워크 정보 사용자가 임의로 지정)
#if defined(WIZ550io_WITH_MACADDRESS) 
  Ethernet.begin(ip, myDns, gateway, subnet);
#else
  Ethernet.begin(mac, ip, myDns, gateway, subnet);
#endif
#endif

  // give the Ethernet shield a second to initialize:
  delay(1000);

  // 서버 연결 시작 (Socket open -> Listen 상태 돌입)
  server.begin();
  // Open serial communications and wait for port to open:
  SerialUSB.begin(9600);
  // WizArduino M0(Zero)는 컴파일할 때 BOOT모드로 변경되었다가 App으로 동작하게 된다. 
  // 이로인해 시리얼을 사용할 때 Delay 6초를 해준다. (업로드 시 MCU가 BOOT로 갔다가 App으로 넘어오는 시간이 6초이기 때문) 
  delay(6000);
  while (!SerialUSB) {
    ; // 시리얼 포트가 연결 될 때까지 기다림, 레오나르도 보드의 경우만에만 필요
  }
  SerialUSB.print("Chat server address:");
  SerialUSB.println(Ethernet.localIP());
  
  pinMode(LED_PORT, OUTPUT);    // LED PORT 출력 설정
  digitalWrite(LED_PORT, LOW); // LED PORT Defualt 'OFF'으로 설정 
}

void loop() {
  EthernetClient client = server.available(); // ESTABLISHED & CLOSE_WAIT 상태인지 확인.
  // server.available()을 반복문으로 돌면서 클라이언트 연결이 올 때까지 대기.
  // 만약 클라이언트로 부터 연결(connect)이 오면 칩 내부적으로 연결(accept)을 하고 ESTBLISHED상태로 넘어간다.
  // ESTABLISHED 상태가 되면 Connection이 되었다는 것을 의미.
  if (client) {
    if (!alreadyConnected) { // Flag를 이용하여 초기에 버퍼 클리어
      client.flush(); // 버퍼 초기화
      SerialUSB.println("We have a new client");
      alreadyConnected = true;
    } 
    if (client.available() > 0) { // 클라이언트로 부터 수신된 데이터가 있다면
      char c = client.read(); // 데이터를 1byte씩 읽어서 thisChar 변수에 저장
      buffer += c;  // 1 byte 데이터를 문자열 버퍼에 저장
      if(buffer.indexOf("led_on")>=0){
        digitalWrite(LED_PORT, HIGH);
        buffer.remove(0); // 0번째 인덱스부터 문자 제거
      }
      else if(buffer.indexOf("led_off")>=0){
        digitalWrite(LED_PORT, LOW);
        buffer.remove(0); // 0번째 인덱스부터 문자 제거
      }
    }
  }
}



